/*
**  CONFTP.C [Requires FCE Ver 2.0.0 or above]
**
**  Console mode FTP client.
*/

#include <windows.h>
#include <winsock.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "fce.h"
#include "keycode.h"

/* globals */

#define BS          8
#define LF         10
#define CR         13
#define ESC        27
#define TAB         9

#define MAX_STR   128
#define MAX_BUF  8192

static char Temp[MAX_STR];
static char CmdBuffer[MAX_STR];
static char ServerDir[MAX_STR];
static char LocalDir[MAX_STR];
static char PathBuffer[MAX_STR];
static char DataBuffer[MAX_BUF];
static char XferMode = 'A';
static int  PassiveMode = 0;

void Help(void)
{printf("Commands are:\n");
 printf("    HELP : Get help\n");
 printf("    QUIT : Quit program\n");
 printf("   ASCII : Set ASCII mode\n");
 printf("  BINARY : Set BINARY mode\n");
 printf("    LIST : List all files\n");
 printf("    NLST : List file names only\n");
 printf("LIST xxx : List file xxx\n");
 printf("NLST xxx : List file xxx (name only)\n");
 printf("     SSD : Set server directory (CWD)\n");
 printf("     DSD : Display server directory (PWD)\n");
 printf("     SLD : Set local directory\n");
 printf("     DLD : Display local directory\n");
 printf(" GET xxx : Get file xxx\n");
 printf(" PUT xxx : Put file xxx\n");
 printf(" DEL xxx : Delete file xxx\n");
 printf(" FLIST n : Display fields for file # n (n>=1)\n");
 printf(" MKD xxx : Make server directory xxx\n");
 printf(" RMD xxx : Remove server directory xxx\n");
 printf("    PASV : Toggle passive mode (default is off)\n");
 printf("   ALIVE : Keep alive\n");
 printf(" CMD xxx : Send arbitary command 'xxx' to server\n");
}

void ShowError(int Code)
{Temp[0] = '\0';
 fceErrorText(0,Code,(LPSTR)Temp,MAX_STR);
 printf("ERROR %d: %s\n", Code, Temp);
}

void ErrorExit(int Code)
{ShowError(Code);
 fceRelease();
 exit(1);
}

void ShowProgress(char Direction)
{/* display progress */
 ULONG ByteCount;
 ULONG LastCount = 0;
 if(Direction=='G') ByteCount = fceGetInteger(0, FCE_GET_FILE_BYTES_RCVD);
 if(Direction=='P') ByteCount = fceGetInteger(0, FCE_GET_FILE_BYTES_SENT);
 if(ByteCount!=LastCount)
   {printf("%d bytes\r",ByteCount);
    LastCount = ByteCount;
   }
}

/*
** CallTheDriver calls fceDriver in a loop until it returns 0,
** indicating that it is done. Refer to the FCE Users Manual
** (FCE4C_U.TXT) for details on using fceDriver.
*/

ULONG CallTheDriver(char Direction)
{int   Code;
 ULONG TimeNow;
 ULONG TimeStart;
 ULONG TimeMark;
 fceSetString(0,FCE_WRITE_TO_LOG,(LPSTR)"--> Entering CallTheDriver");
 /* get start time */
 TimeStart = GetCurrentTime();
 TimeMark = TimeStart + 1000;
 /* AUTO_CALL_DRIVER assumed set on (1) */
 while(1)
   {Code = fceDriver(0);
    if(Code<0) ShowError(Code);
    if(Code<=0) break;
    /* check progress every second */
    TimeNow = GetCurrentTime();
    if(TimeNow>=TimeMark)
      {TimeMark = TimeNow + 1000;
       /* display progress */
       ShowProgress(Direction);
      }
   }
 /* compute run time */
 TimeNow = GetCurrentTime() - TimeStart;
 /* re-enable automatic call to seeDriver */
 fceSetInteger(0,FCE_SET_AUTO_CALL_DRIVER,1);
 fceSetString(0,FCE_WRITE_TO_LOG,(LPSTR)"--> Leaving CallTheDriver");
 ShowProgress(Direction);
 return TimeNow;
}

/* read string from keyboard */

void GetString(LPSTR Buffer)
{char c;
 ///int  n;
 int  Flag = 0;
 /* get rest of line */
 while(1)
   {scanf("%c",&c);
    if((c=='\n')||(c=='\r'))
      {*Buffer++ = '\0';
       return;
      }
    if((Flag==0)&&(c==' ')) ; // skip leading spaces
    else
      {Flag = 1;
       *Buffer++ = c;
      }
   }
}

int FileExists(LPSTR Filename)
{int Handle;
 lstrcpy(PathBuffer,(LPSTR)LocalDir);
 lstrcat(PathBuffer,(LPSTR)"\\");
 lstrcat(PathBuffer,(LPSTR)Filename);
 Handle = _lopen((LPSTR)PathBuffer,OF_READ);
 if(Handle==-1) return FALSE;
 _lclose(Handle);
 return TRUE;
}

void main(int argc, char *argv[])
{int i, n, Code;
 LPSTR ServerPtr;
 LPSTR UserPtr;
 LPSTR PassPtr;
 ULONG ByteCount;
 ULONG TheTime;
 int BufLen;
 int Version;
 int Build;
 /* check arguments */
 if(argc!=4)
   {printf("Usage: CONFTP server user password \n");
    printf("   Eg: CONFTP ftp.marshallsoft.com anonymous you@yourisp.com\n");
    exit(1);
   }

 printf("CONFTP 1/3/2000\n");
 ServerPtr = (LPSTR) argv[1];
 UserPtr = (LPSTR) argv[2];
 PassPtr = (LPSTR) argv[3];
 lstrcpy(ServerDir,".\\");

 Code = fceAttach(1, FCE_KEY_CODE);
 if(Code<0) ErrorExit(Code);
 Version = fceGetInteger(0,FCE_GET_VERSION);
 Build   = fceGetInteger(0,FCE_GET_BUILD);
 printf("FCE32 Version: %1d.%1d.%1d Build %d\n",
    0x0f&(Version>>8),0x0f&(Version>>4),0x0f&Version,Build);
 fceGetString(0,FCE_GET_REGISTRATION,(LPSTR)Temp,50);
 printf(" Registration: %s\n", Temp);
 Code = fceSetString(0,FCE_SET_LOG_FILE,(LPSTR)"conftp.log");

 printf("Connecting to '%s'\n", ServerPtr);

 Code = fceConnect(0,ServerPtr,UserPtr,PassPtr);
 if(Code<0) ErrorExit(Code);

 fceGetString(0, FCE_GET_SERVER_IP, (LPSTR)Temp, 30);
 printf("Connected to %s\n", (LPSTR)Temp);
 printf("Type HELP for list of commands.\n");
 /* ready for user commands */
 while(1)
     {printf(">");
      n = scanf("%s",CmdBuffer);
      ///printf("%d[%s]\n", n, CmdBuffer);
      /* force command to upper case */
      BufLen = (int)strlen(CmdBuffer);
      for(i=0;i<BufLen;i++) CmdBuffer[i] = toupper(CmdBuffer[i]);

      /* QUIT */

      if( (strcmp(CmdBuffer,"QUIT")==0) ||
          (strcmp(CmdBuffer,"Q")==0))
        {/* QUIT */
         fceClose(0);
         fceRelease();
         break;
        }

      /* Set Server Directory */

      if( (strcmp(CmdBuffer,"SSD")==0)||
          (strcmp(CmdBuffer,"CWD")==0)||
          (strcmp(CmdBuffer,"CD")==0))
         {//n = scanf("%s",ServerDir);
          GetString((LPSTR)ServerDir);
          Code = fceSetServerDir(0, (LPSTR)ServerDir);
          if(Code<0) ShowError(Code);
          continue;
         }

      /* Display (list) Server Directory */

      if( (strcmp(CmdBuffer,"DSD")==0)||
          (strcmp(CmdBuffer,"PWD")==0))
         {Code = fceGetServerDir(0, (LPSTR)ServerDir, MAX_STR);
          if(Code<0) ShowError(Code);
          else printf("%s\n", ServerDir);
          continue;
         }

      /* Set Local Directory */

      if(strcmp(CmdBuffer,"SLD")==0)
         {GetString((LPSTR)LocalDir);
          /* does local directory exist ? */
          if(GetCurrentDirectory(MAX_STR,(LPSTR)Temp)>0)
             {if(!SetCurrentDirectory((LPSTR)LocalDir))
                {printf("Directory %s does not exist\n",LocalDir);
                 LocalDir[0] = '\0';
                 continue;
                }
              /* restore current directory */
              SetCurrentDirectory((LPSTR)Temp);
             }
          /* set local directory for transfers */
          Code = fceSetLocalDir(0, (LPSTR)LocalDir);
          if(Code<0) ShowError(Code);
          continue;
         }

      /* Display Local Directory */

      if(strcmp(CmdBuffer,"DLD")==0)
         {Code = fceGetLocalDir(0, (LPSTR)LocalDir, MAX_STR);
          if(Code<0) ShowError(Code);
          else printf("%s\n", LocalDir);
          continue;
         }

      /* Set ASCII Xfer Mode */

      if(strcmp(CmdBuffer,"ASCII")==0)
         {Code = fceSetMode(0,'A');
          if(Code<0) ShowError(Code);
          XferMode = 'A';
          continue;
         }

      /* Set BINARY Xfer Mode */

      if(strcmp(CmdBuffer,"BINARY")==0)
         {Code = fceSetMode(0,'B');
          if(Code<0) ShowError(Code);
          XferMode = 'B';
          continue;
         }

      /* List Files */

      if(strcmp(CmdBuffer,"LIST")==0)
         {DataBuffer[0] = '\0';
          GetString((LPSTR)DataBuffer);
          if(DataBuffer[0]==0)
            Code = fceGetList(0,FCE_FULL_LIST,(LPSTR)DataBuffer,MAX_BUF);
          else
            Code = fceGetList(0,FCE_FULL_LIST_FILE,(LPSTR)DataBuffer,MAX_BUF);
          if(Code>0)
            {int Lines;
             // count the lines (linefeeds) in DataBuffer
             Lines = fceGetString(0, FCE_GET_LINE_COUNT, (LPSTR)DataBuffer, MAX_BUF);
             printf("%s\n%d lines (%d bytes)\n", DataBuffer, Lines, Code);
            }
          else printf("File not found.\n");
          continue;
         }

      /* List Files by Name (only) */

      if((strcmp(CmdBuffer,"NLIST")==0)||
         (strcmp(CmdBuffer,"LS")==0)||
         (strcmp(CmdBuffer,"NLST")==0))
         {DataBuffer[0] = '\0';
          GetString((LPSTR)DataBuffer);
          if(DataBuffer[0]==0)
            Code = fceGetList(0,FCE_NAME_LIST,(LPSTR)DataBuffer,MAX_BUF);
          else
            Code = fceGetList(0,FCE_NAME_LIST_FILE,(LPSTR)DataBuffer,MAX_BUF);
          if(Code>0) printf("%s\n", DataBuffer);
          else printf("File not found.\n");
          continue;
         }

      /* Help */

      if(strcmp(CmdBuffer,"HELP")==0)
        {Help();
         continue;
        }

      /* Get (download) File from Server */

      if(strcmp(CmdBuffer,"GET")==0)
        {GetString((LPSTR)DataBuffer);
         printf("Getting '%s'\n", DataBuffer);
         printf("Xfer mode is %c\n", XferMode);
         /* disable automatic call to fceDriver */
         fceSetInteger(0,FCE_SET_AUTO_CALL_DRIVER,0);
         /* ask for the file */
         Code = fceGetFile(0,(LPSTR)DataBuffer);
         if(Code<0) ErrorExit(Code);
         TheTime = CallTheDriver('G');
         /* display final count */
         ByteCount = fceGetInteger(0, FCE_GET_FILE_BYTES_RCVD);
         printf("%d bytes received in %0.2f seconds\n",ByteCount,(float)TheTime/1000.0);
         continue;
        }

      /* Put (upload) File to Server */

      if(strcmp(CmdBuffer,"PUT")==0)
        {GetString((LPSTR)DataBuffer);
         if(FileExists((LPSTR)DataBuffer))
           {printf("Putting '%s'\n", DataBuffer);
            printf("Xfer mode is %c\n", XferMode);
            /* disable automatic call to fceDriver */
            fceSetInteger(0,FCE_SET_AUTO_CALL_DRIVER,0);
            Code = fcePutFile(0,(LPSTR)DataBuffer);
            if(Code<0) ErrorExit(Code);
            TheTime = CallTheDriver('P');
            /* display final count */
            ByteCount = fceGetInteger(0, FCE_GET_FILE_BYTES_SENT);
            printf("%d bytes sent in %0.2f seconds\n",ByteCount,(float)TheTime/1000.0);
           }
         else printf("File %s does not exist\n", (LPSTR)DataBuffer);
         continue;
        }

      /* Delete file on Server */

      if(strcmp(CmdBuffer,"DEL")==0)
         {GetString((LPSTR)DataBuffer);
          printf("Deleting '%s'\n", DataBuffer);
          Code = fceDelFile(0,(LPSTR)DataBuffer);
          if(Code<0) ShowError(Code);
          continue;
         }

      /* Field List */

      if(strcmp(CmdBuffer,"FLIST")==0)
         {int Total = 0;
          char LineBuf[100];
          DataBuffer[0] = '\0';
          //scanf("%s",DataBuffer);
          GetString((LPSTR)DataBuffer);
          n = atoi(DataBuffer);
          if(n<=0)
            {printf("ERROR: No such line %d\n",n);
             continue;
            }
          /* ask for full list */
          DataBuffer[0] = '\0';
          Code = fceGetList(0,FCE_FULL_LIST,(LPSTR)DataBuffer,MAX_BUF);
          if(Code<0) ShowError(Code);
          /* get each field of line n */
          for(i=1;i<=9;i++)
            {Code = fceExtract((LPSTR)DataBuffer, n, i, (LPSTR)LineBuf, 100);
             Total += Code;
             if(Code>0) printf("LINE %d, FIELD %d: Len %2d [%s] \n", n, i, Code, LineBuf);
            }
          if(Total==0) printf("ERROR: No data\n");
          continue;
         }

      /* Make Directory on Server */

      if(strcmp(CmdBuffer,"MKD")==0)
         {GetString((LPSTR)ServerDir);
          Code = fceMakeServerDir(0, (LPSTR)ServerDir);
          if(Code<0) ShowError(Code);
          continue;
         }

      /* Remove (delete) Directory on Server */

      if(strcmp(CmdBuffer,"RMD")==0)
         {GetString((LPSTR)ServerDir);
          Code = fceDelServerDir(0, (LPSTR)ServerDir);
          if(Code<0) ShowError(Code);
          continue;
         }

      /* Toggle "Passive" Mode */

      if(strcmp(CmdBuffer,"PASV")==0)
         {if(PassiveMode)
            {printf("Disabling PASSIVE mode\n");
             PassiveMode = 0;
            }
          else
            {printf("Enabling PASSIVE mode\n");
             PassiveMode = 1;
            }
          fceSetInteger(0, FCE_SET_PASSIVE, PassiveMode);
          continue;
         }

      /* keep alive */

      if(strcmp(CmdBuffer,"ALIVE")==0)
         {Code = fceHello(0);
          if(Code<0) ShowError(Code);
          ///lstrcpy((LPSTR)Temp,"xxx.xxx");
          ///Code = fceGetList(0,FCE_NAME_LIST_FILE,(LPSTR)Temp,MAX_BUF);
          ///if(Code<0) ShowError(Code);
          continue;
         }

      if(strcmp(CmdBuffer,"CMD")==0)
        {GetString((LPSTR)DataBuffer);
         printf("Sending command '%s'\n", DataBuffer);
         Code = fceCommand(0,(LPSTR)DataBuffer);
         if(Code<0) ShowError(Code);
         Code = fceGetInteger(0, FCE_GET_RESPONSE);
         printf("Server response was %d\n", Code);

         Code = fceGetString(0, FCE_GET_LAST_RESPONSE, (LPSTR)Temp,50);
         if(Code>0) printf("%s", Temp);

         continue;
        }

      /* don't recognize commands */

      printf("Cannot recognize command %s\n", CmdBuffer);
      printf("Type HELP for help\n");
     }
}

