/*
**  DIAL.C
**
**  Console mode DUN dialer.
**
**  To compile: nmake -f dial._M_
*/

#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "mdd.h"
#include "keycode.h"

void DisplayError(int Code)
{int Len;
 char Buffer[128];
 Len = mddErrorText(Code, (LPSTR)Buffer, 128);
 if(Len>0) printf("ERROR %d: %s\n", Code, Buffer);
}

void ErrorExit(int Code)
{DisplayError(Code);
 mddRelease();
 exit(1);
}

void main(int argc, char *argv[])
{int i, Code;
 int Status;
 int Version;
 int Build;
 int NbrEntries;
 int LastStatus = 0;
 int DialEntry;
 char Buffer[128];
 printf("MarshallSoft DUN Dialer (MDD) Example\n");
 if(argc!=2)
   {printf("Usage: DIAL <entry #>\n");
    printf("   Eg: DIAL 0  <-- dial 1st entry\n");
    printf("   Eg: DIAL 1  <-- dial 2nd entry\n");
    exit(1);
   }
 DialEntry = atoi(argv[1]);
 Code = mddAttach(MDD_KEY_CODE);
 if(Code<0)
   {printf("ERROR %d: Cannot attach MDD. Check your key code.\n", Code);
    exit(1);
   }
 /* display build & version */
 Version = mddDebug(MDD_GET_VERSION,(LPSTR)Buffer,65);
 Build = mddDebug(MDD_GET_BUILD,(LPSTR)Buffer,65);
 printf("MDD32 Version: %1d.%1d.%1d Build %d\n",
   0x0f&(Version>>8),0x0f&(Version>>4),0x0f&Version,Build);
 mddDebug(MDD_GET_REGISTRATION,(LPSTR)Buffer,65);
 printf(" Registration: %s\n", Buffer);
 /* load all dial entries */
 Code = mddLoadEntries();
 if(Code<0) ErrorExit(Code);
 printf("%d available entry(s) [First entry is #0].\n", Code);
 if(Code==0) exit(0);
 /* good entry number ? */
 NbrEntries = Code;
 if(DialEntry>=NbrEntries)
   {printf("ERROR: Entry %d does not exist!\n",DialEntry);
    mddRelease();
    exit(1);
   }
 /* list all dialing entries */
 for(i=0;i<NbrEntries;i++)
   {Buffer[0] = '\0';
    Code = mddGetEntry(i, (LPSTR)Buffer, 128);
    if(Code<0) ErrorExit(Code);
    printf("Entry %d [%s]\n", i, Buffer);
   }
 /* ready to dial */
 printf("===[ Dialing using entry %d ]===\n",DialEntry);
 Code = mddBeginDial(DialEntry);
 if(Code<0) ErrorExit(Code);
 while(1)
   {/* get dialing status */
    Status = mddDialStatus();
    if(Status<0) ErrorExit(Status);
    if(Status!=LastStatus)
      {LastStatus = Status;
       /* display status */
       Code = mddStatusText(Status,(LPSTR)Buffer, 128);
       if(Code>0) printf("Status %4d: %s\n", Status, Buffer);
       if( (LastStatus==MDD_IS_CONNECTED) ||
           (LastStatus==MDD_IS_DISCONNECTED)
         ) break;
      }
    Sleep(100);
   }
 /* are we connected ? */
 printf("Last status is %d\n", LastStatus);
 if(LastStatus==MDD_IS_CONNECTED)
   {printf("Connected! Type any character to hang up: ");
    /* wait for user to type CR */
    scanf("%s",Buffer);
   }
 /* all done, release MDD */
 printf("Disconnecting.\n");
 mddHangup();
 mddRelease();
}
