/*
**  DMAN.C
**
**  Console mode DUN dialer manager.
**
**  To compile: nmake -f dman._M_
*/

#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "mdd.h"
#include "keycode.h"

static int NbrEntries;

void DisplayError(int Code)
{char Buffer[128];
 if(mddErrorText(Code, (LPSTR)Buffer, 128) > 0)
   printf("ERROR %d: %s\n", Code, Buffer);
}

void ErrorExit(int Code)
{DisplayError(Code);
 mddRelease();
 exit(1);
}

void Dialer(int Entry)
{int Code;
 int Status;
 int LastStatus = 0;
 char Buffer[128];
 FILE *FilePtr;
 Code = mddBeginDial(Entry);
 if(Code<0) ErrorExit(Code);
 Buffer[0] = '\0';
 while(1)
   {///printf(".");
    /* get dialing status */
    Status = mddDialStatus();
    if(Status<0) ErrorExit(Status);
    if(Status!=LastStatus)
      {LastStatus = Status;
       /* display status */
       Code = mddStatusText(Status,(LPSTR)Buffer, 128);
       printf("Status %4d: %s\n", Status, Buffer);
       if( (LastStatus==MDD_IS_CONNECTED) ||
           (LastStatus==MDD_IS_DISCONNECTED) ||
           (LastStatus==MDD_IS_AUTHENTICATED)
         ) break;
      }
    Sleep(1000);
   }
 FilePtr = fopen("STATUS.OUT","wt");
 if(FilePtr)
   {char Temp[128];
    sprintf(Temp,"%d %s\0",LastStatus,Buffer);
    fwrite(Temp,strlen(Temp),1,FilePtr);
    fclose(FilePtr);
   }
}

void main(void)
{int i, Code;
 int Version;
 int Build;
 int Entry = 0;
 char Buffer[132];
 char Temp[140];
 FILE *FilePtr;
 printf("MarshallSoft DUN Dialer (MDD) Dial Manager\n");
 if(SetCurrentDirectory("Control")==0)
   {printf("Cannot set 'control' as current directory.\n");
    exit(1);
   }
 Code = mddAttach(MDD_KEY_CODE);
 if(Code<0)
   {printf("ERROR %d: Cannot attach MDD. Check your key code.\n", Code);
    exit(1);
   }
 /* display build & version */
 Version = mddDebug(MDD_GET_VERSION,(LPSTR)Buffer,65);
 Build = mddDebug(MDD_GET_BUILD,(LPSTR)Buffer,65);
 printf("MDD32 Version: %1d.%1d.%1d Build %d\n",
   0x0f&(Version>>8),0x0f&(Version>>4),0x0f&Version,Build);
 mddDebug(MDD_GET_REGISTRATION,(LPSTR)Buffer,65);
 printf(" Registration: %s\n", Buffer);
 /* load all dial entries */
 Code = mddLoadEntries();
 if(Code<0) ErrorExit(Code);
 printf("%d available entries.\n", Code);
 if(Code==0) exit(0);
 FilePtr = fopen("ENTRIES.OUT","wt");
 /* list all dialing entries */
 NbrEntries = Code;
 for(i=0;i<NbrEntries;i++)
   {Buffer[0] = '\0';
    Code = mddGetEntry(i, (LPSTR)Buffer, 128);
    if(Code<0) ErrorExit(Code);
    Buffer[Code] = '\0';
    sprintf(Temp,"%d [%s]\n", i, Buffer);
    if(FilePtr) fwrite(Temp,strlen(Temp),1,FilePtr);
   }
  if(FilePtr) fclose(FilePtr);
  /* monitor the CONTROL directory */
  while(1)
    {printf(".");
     FilePtr = fopen("DIAL.CMD","rt");
     if(FilePtr)
       {Code = fscanf(FilePtr,"%d",&Entry);
        fclose(FilePtr);
        DeleteFile("DIAL.CMD");
        if(Code<0)
          {printf("Error reading DIAL.CMD\n");
           exit(1);
          }
        if((Entry<0)||(Entry>=NbrEntries))
          {printf("Bad entry number (%d) in DIAL.CMD\n",Entry);
           break;
          }
        DeleteFile("STATUS.OUT");
        printf("Dialing entry %d\n", Entry);
        Dialer(Entry);
       }
     FilePtr = fopen("HANGUP.CMD","rt");
     if(FilePtr)
       {fclose(FilePtr);
        DeleteFile("HANGUP.CMD");
        DeleteFile("STATUS.OUT");
        printf("Hanging up.\n");
        mddHangup();
       }
     FilePtr = fopen("EXIT.CMD","rt");
     if(FilePtr)
       {fclose(FilePtr);
        DeleteFile("EXIT.CMD");
        printf("Exiting.\n");
        mddHangup();
        mddRelease();
        exit(0);
       }
     Sleep(3000);
    }
 }