/* files.c */

#include <windows.h>
#include "fce.h"
#include "files.h"
#include "info.h"
#include "message.h"

extern HWND hMainWnd;            /* main window handle */
extern HWND hInfoWnd;            /* popup handle */ 
extern HCURSOR ArrowCursor;      /* arrow cursor */
extern HCURSOR WaitCursor;
extern int ConnectStatus;

#define USE_FCE_DRIVER 
#define KEEP_ALIVE_INTERVAL 115000

#define MAX_BUF 10000
#define MAX_STR 64

/* private */

static HWND hDlgWnd = 0;
static int  PassiveMode = 0;
static int  NextState = STATE_IDLE;
static int  BinaryMode = FALSE;
static char DataBuffer[MAX_BUF];
static char ServerDir[MAX_STR];
static char LocalDir[MAX_STR];
static char FileName[MAX_STR];
static char Temp[MAX_STR+MAX_STR];

void Message(LPSTR);
int AskUser(LPSTR);
void ShowError(int,LPSTR);
int IsConnected(void);

void SetStatusText(HWND hDlg, LPSTR Text)
{
 SetDlgItemText(hDlg, ID_STAT_BOX, Text);
}

void AddStatusText(HWND hDlg, LPSTR Text)
{int Len;
 Len = GetDlgItemText(hDlg, ID_STAT_BOX, (LPSTR)DataBuffer, MAX_BUF-MAX_STR);
 if(Len>0)
   {DataBuffer[Len] = '\r';
    DataBuffer[Len+1] = '\n';
    lstrcpy((LPSTR)&DataBuffer[Len+2],Text);
    SetDlgItemText(hDlg, ID_STAT_BOX, (LPSTR)DataBuffer);
   }
}

void DisplayError(int Code)
{char Buffer[100];
 ShowError(Code, (LPSTR)Buffer);
 if(hDlgWnd) AddStatusText(hDlgWnd,(LPSTR)Buffer); 
}

void EnableStopButton(HWND hDlg, int EnableFlag)
{int i;
 EnableWindow(GetDlgItem(hDlg, ID_STOP_BTN), EnableFlag);
 for(i=ID_EXIT_BTN;i<=ID_CHECK_BTN;i++) 
    EnableWindow(GetDlgItem(hDlg,i), !EnableFlag);
} 

int FileExists(LPSTR PathBuffer)
{int Handle;
 lstrcpy(PathBuffer,(LPSTR)LocalDir);
 lstrcat(PathBuffer,(LPSTR)"\\");
 lstrcat(PathBuffer,(LPSTR)FileName);
 Handle = _lopen((LPSTR)PathBuffer,OF_READ);
 if(Handle==-1) return FALSE;
 _lclose(Handle);
 return TRUE;
}

int MustBeIdle(void)
{if(NextState==STATE_IDLE) return TRUE;
 Message((LPSTR)"Operation in progess."); 
 return FALSE;
}

#ifdef WIN32
BOOL CALLBACK
#else
BOOL FAR PASCAL
#endif
FilesDlgProc(HWND hDlg, UINT iMsg, WPARAM wParam, LPARAM lParam)
{int i, Code;
 static long ByteCount;
 static long LastCount;
 static DWORD KeepAliveTimeMark;
 switch (iMsg)
   {case WM_INITDIALOG:
      hDlgWnd = hDlg;
      if(IsConnected()) 
        {/* show local directory */        
         Code = fceGetLocalDir(0, (LPSTR)LocalDir, MAX_STR);    
         if(Code<0) DisplayError(Code);
         else
           {LocalDir[Code] = '\0';
            SetDlgItemText(hDlg, ID_LDIR_BOX, (LPSTR)LocalDir);
            wsprintf((LPSTR)Temp,"Local directory is '%s'", (LPSTR)LocalDir);
            SetStatusText(hDlg,(LPSTR)Temp);
           }          
         /* get server directory */
         fceGetServerDir(0,(LPSTR)ServerDir, MAX_STR);
         SetDlgItemText(hDlg, ID_SDIR_BOX, (LPSTR)ServerDir);
         wsprintf((LPSTR)Temp,"Server directory is '%s'", (LPSTR)ServerDir);
         AddStatusText(hDlg,(LPSTR)Temp); 
         /* set ASCII mode */
         Code = fceSetMode(0,'A');
         if(Code<0) DisplayError(Code);
         else
           {BinaryMode = FALSE;
            CheckRadioButton(hDlg, ID_ASC_RB, ID_BIN_RB, ID_ASC_RB);
            AddStatusText(hDlg,"ASCII mode set");
           }
         SetTimer(hDlgWnd,0,1000,NULL);
         /* do "keep alive" 'KEEP_ALIVE_INTERVAL' seconds */
         KeepAliveTimeMark = GetCurrentTime() + KEEP_ALIVE_INTERVAL;
        }
      else EndDialog(hDlg, 0);
      /* disable STOP button */
      EnableStopButton(hDlg, 0);
      /* enable passive mode */
      ///fceSetInteger(0, FCE_SET_PASSIVE, 1);
      return TRUE;

    case WM_TIMER:
      switch(NextState)
        {case STATE_IDLE:
           if(GetCurrentTime() > KeepAliveTimeMark)
             {KeepAliveTimeMark = GetCurrentTime() + KEEP_ALIVE_INTERVAL;
              if(!MustBeIdle()) return TRUE;
              Code = fceHello(0);
              if(Code<0) DisplayError(Code);
              /* do transfer to prevent being timed out by the server */
              lstrcpy((LPSTR)Temp,"xxx.xxx");
              if(!MustBeIdle()) return TRUE;
              Code = fceGetList(0, FCE_NAME_LIST_FILE,(LPSTR)Temp, MAX_STR);  
              if(Code<0) DisplayError(Code);
             }
           return TRUE;
           
         case STATE_UPLOAD:
           NextState = STATE_IDLE;
           for(i=1;i<=5;i++)
              {Code = fceDriver(0);
               if(Code<0) 
                  {DisplayError(Code);
                   NextState = STATE_CLEANUP;
                   return TRUE;
                  } 
               if(Code==0) 
                 {ByteCount = fceGetInteger(0, FCE_GET_FILE_BYTES_SENT);
                  InfoFileMsg(ByteCount);
                  MessageBeep(0);
                  wsprintf((LPSTR)Temp,"File '%s' uploaded (%ld bytes)",
                                    (LPSTR)FileName, ByteCount);
                  AddStatusText(hDlg,(LPSTR)Temp);   
                  NextState = STATE_CLEANUP;
                  return TRUE;
                 }
              }
           /* display progress */
           ByteCount = fceGetInteger(0, FCE_GET_FILE_BYTES_SENT);
           if(ByteCount!=LastCount) 
             {InfoFileMsg(ByteCount);
              ShowWindow(hInfoWnd,SW_SHOW);
              SendMessage(hInfoWnd,WM_USER,0,0L);
              LastCount = ByteCount;
             }               
           NextState = STATE_UPLOAD;  
           break;
           
         case STATE_DNLOAD:
           NextState = STATE_IDLE;
           for(i=1;i<=7;i++)
             {Code = fceDriver(0);
              if(Code<0) 
                {DisplayError(Code);
                 NextState = STATE_CLEANUP;
                 return TRUE;
                } 
              if(Code==0) 
                {NextState = STATE_CLEANUP; 
                 ByteCount = fceGetInteger(0, FCE_GET_FILE_BYTES_RCVD);
                 InfoFileMsg(ByteCount);
                 MessageBeep(0);
                 wsprintf((LPSTR)Temp,"File '%s' downloaded (%ld bytes)",
                                     (LPSTR)FileName, ByteCount);
                 AddStatusText(hDlg,(LPSTR)Temp);
                 NextState = STATE_CLEANUP;
                 return TRUE;
                }       
             }
          /* display progress */
          ByteCount = fceGetInteger(0, FCE_GET_FILE_BYTES_RCVD);
          if(ByteCount!=LastCount) 
            {InfoFileMsg(ByteCount);
             ShowWindow(hInfoWnd,SW_SHOW);
             SendMessage(hInfoWnd,WM_USER,0,0L);
             LastCount = ByteCount;
            }              
          NextState = STATE_DNLOAD;
          break;
          
        case STATE_CLEANUP:
          NextState = STATE_IDLE;
          /* re-enable automatic call to seeDriver */
          fceSetInteger(0,FCE_SET_AUTO_CALL_DRIVER,1);
          /* drop info window */
          ShowWindow(hInfoWnd,SW_HIDE);
          SetCursor(ArrowCursor);  
          /* disable STOP button  */
          EnableStopButton(hDlg, 0);
          break;
          
        default:
          NextState = STATE_IDLE;
          
       } /* end-switch */
      return TRUE;
       
    case WM_COMMAND:
      switch (LOWORD(wParam))
        {case ID_EXIT_BTN:
           if(AskUser("Log off server?")!=IDOK) return TRUE;
           if(!IsConnected()) return TRUE;
           ConnectStatus = FALSE;
           fceClose(0);
           KillTimer(hDlgWnd,0);
           EndDialog(hDlg, 0);
           break;
           
        case ID_STOP_BTN:
           NextState = STATE_IDLE;
           fceAbort(0);
           NextState = STATE_CLEANUP;
           Message("Aborted by User!");
           /* disable STOP button */
           EnableStopButton(hDlg, 0);
           break;
           
        case ID_PASSIVE_BTN:
           if(!MustBeIdle()) break;
           if(PassiveMode)
             {AddStatusText(hDlg,"Disabling PASSIVE mode");
              PassiveMode = 0;
             }
           else
             {AddStatusText(hDlg,"Enabling PASSIVE mode");
              PassiveMode = 1;
             }
           fceSetInteger(0, FCE_SET_PASSIVE, PassiveMode);  
           break;
           
        case ID_CLEAR_BTN:
           /* clear display window */
           SetStatusText(hDlg,(LPSTR)" ");
           break;           
           
        case ID_FULL_BTN:
           if(!MustBeIdle()) break;
           /* get full directory list */
           if(!IsConnected()) 
             {EndDialog(hDlg, 0);
              break;
             }
           SetCursor(WaitCursor);
           /* change to ASCII mode */
           if(BinaryMode) fceSetMode(0,'A');
           /* get full file list */
           Code = fceGetList(0,FCE_FULL_LIST,(LPSTR)DataBuffer,MAX_BUF);
           if(Code>=0) SetStatusText(hDlg, (LPSTR)DataBuffer);
           if(BinaryMode) fceSetMode(0,'B');
           SetCursor(ArrowCursor);
           break;
           
         case ID_NAME_BTN:
           if(!MustBeIdle()) break;
           /* get directory name list */
           if(!IsConnected()) 
             {EndDialog(hDlg, 0);
              break;
             }
           SetCursor(WaitCursor);
           /* change to ASCII mode */
           if(BinaryMode) fceSetMode(0,'A');
           /* get filename list */
           Code = fceGetList(0,FCE_NAME_LIST,(LPSTR)DataBuffer,MAX_BUF);
           if(Code>=0) SetStatusText(hDlg, (LPSTR)DataBuffer);
           if(BinaryMode) fceSetMode(0,'B');
           SetCursor(ArrowCursor);
           break;   
           
         case ID_ASC_RB:
           if(!MustBeIdle()) break;
           /* set ASCII mode */
           if(!IsConnected()) 
             {EndDialog(hDlg, 0);
              break;
             }          
           Code = fceSetMode(0,'A');
           if(Code<0) DisplayError(Code);
           else 
             {BinaryMode = FALSE;
              CheckRadioButton(hDlg, ID_ASC_RB, ID_BIN_RB, ID_ASC_RB);
              AddStatusText(hDlg,"ASCII mode set");
             } 
           break;
           
         case ID_BIN_RB:
           if(!MustBeIdle()) break;
           /* set BINARY mode */
           if(!IsConnected()) 
             {EndDialog(hDlg, 0);
              break;
             }          
           Code = fceSetMode(0,'B');
           if(Code<0) DisplayError(Code);
           else 
             {BinaryMode = TRUE;
              CheckRadioButton(hDlg, ID_ASC_RB, ID_BIN_RB, ID_BIN_RB);
              AddStatusText(hDlg,"BINARY mode set");
             }            
           break;
           
         case ID_SDIR_BTN:
           if(!MustBeIdle()) break;
           if(!IsConnected()) 
             {EndDialog(hDlg, 0);
              return TRUE;
             }         
           /* set server directory */
           if(GetDlgItemText(hDlg, ID_SDIR_BOX, (LPSTR)ServerDir, MAX_STR)==0)
             Message("Missing directory name");
           else
             {Code = fceSetServerDir(0,(LPSTR)ServerDir);
              if(Code<0) DisplayError(Code);
              fceGetServerDir(0,(LPSTR)ServerDir, MAX_STR);
              wsprintf((LPSTR)Temp,"Server directory is '%s'",(LPSTR)ServerDir);
              AddStatusText(hDlg,(LPSTR)Temp);
              SetDlgItemText(hDlg, ID_SDIR_BOX, (LPSTR)ServerDir);                
             }
           break;
           
         case ID_LDIR_BTN:
           if(!IsConnected()) 
             {EndDialog(hDlg, 0);
              break;
             }         
           /* set local directory */
           if(GetDlgItemText(hDlg, ID_LDIR_BOX, (LPSTR)LocalDir, MAX_STR)==0)
             Message("Missing directory name");
           else
             {
#ifdef WIN32
              if(GetCurrentDirectory(MAX_STR,(LPSTR)Temp)>0)
                {if(!SetCurrentDirectory((LPSTR)LocalDir))
                   {Message("Local directory does not exist");
                    LocalDir[0] = '\0';
                    break;
                   }
                 /* restore current directory */
                 SetCurrentDirectory((LPSTR) Temp);
                }            
#endif
              Code = fceSetLocalDir(0,(LPSTR)LocalDir);
              if(Code<0) DisplayError(Code);
              else 
                {wsprintf((LPSTR)Temp,"Local directory is '%s'",(LPSTR)LocalDir);
                 AddStatusText(hDlg,(LPSTR)Temp);
                }              
             }
           break; 
           
         case ID_GET_BTN:
           if(!MustBeIdle()) break;
           if(!IsConnected()) 
             {EndDialog(hDlg, 0);
              break;
             }  
           /* download file */
           if(GetDlgItemText(hDlg, ID_FILE_BOX, (LPSTR)FileName, MAX_STR)==0)
             Message("Missing file name");
           else
             {/* enable STOP button */
              EnableStopButton(hDlg, 1);
              /* download the file */
              SetCursor(WaitCursor);
              /* disable AUTO */
              fceSetInteger(0,FCE_SET_AUTO_CALL_DRIVER,0);
              Code = fceGetFile(0,(LPSTR)FileName);            
              if(Code<0) DisplayError(Code);
              else NextState = STATE_DNLOAD;
             }
           break; 
           
         case ID_PUT_BTN:
           if(!MustBeIdle()) break;
           if(!IsConnected()) 
             {EndDialog(hDlg, 0);
              break;
             } 
           /* upload file */
           if(GetDlgItemText(hDlg, ID_FILE_BOX, (LPSTR)FileName, MAX_STR)==0)
             Message("Missing file name");
           else
             {/* does file exist ? */
              char PathBuffer[MAX_STR];
              if(!FileExists((LPSTR)PathBuffer))
                {wsprintf((LPSTR)Temp,"%s does not exist",(LPSTR)PathBuffer);
                 AddStatusText(hDlg,(LPSTR)Temp);
                 break;
                }
              /* enable STOP button */
              EnableStopButton(hDlg, 1);
              /* upload the file */
              SetCursor(WaitCursor);
              /* disable AUTO */
              fceSetInteger(0,FCE_SET_AUTO_CALL_DRIVER,0);
              Code = fcePutFile(0,(LPSTR)FileName);
              if(Code<0) 
                {DisplayError(Code);
                 NextState = STATE_CLEANUP;
                 return TRUE;
                } 
              else NextState = STATE_UPLOAD;
             }
           break;   
           
         case ID_DEL_BTN:
           if(!MustBeIdle()) break;
           if(!IsConnected()) 
             {EndDialog(hDlg, 0);
              break;
             }  
           /* delete file */
           if(GetDlgItemText(hDlg, ID_FILE_BOX, (LPSTR)FileName, MAX_STR)==0)
             Message("Missing file name");
           else
             {/* delete the file */
              SetCursor(WaitCursor);
              Code = fceDelFile(0,(LPSTR)FileName);             
              SetCursor(ArrowCursor);
              /* check status of deletion */
              if(Code<0) DisplayError(Code);
              else 
                {wsprintf((LPSTR)Temp,"File '%s' deleted",(LPSTR)FileName);
                 AddStatusText(hDlg,(LPSTR)Temp);
                }
             }
           break;
           
         case ID_CHECK_BTN:
           if(!MustBeIdle()) break;
           if(!IsConnected()) 
             {EndDialog(hDlg, 0);
              break;
             }  
           /* check existance of file */
           if(GetDlgItemText(hDlg, ID_FILE_BOX, (LPSTR)FileName, MAX_STR)==0)
             Message("Missing file name");
           else
             {/* check the file */
              SetCursor(WaitCursor);
              strcpy(Temp, FileName);
              Code = fceGetList(0,FCE_NAME_LIST_FILE,(LPSTR)Temp,MAX_STR);                         
              SetCursor(ArrowCursor);
              /* check status of deletion */
              if(Code<0) DisplayError(Code);
              if(Code>0)wsprintf((LPSTR)Temp,"File '%s' found",(LPSTR)FileName);
              else wsprintf((LPSTR)Temp,"File '%s' not found",(LPSTR)FileName);
              AddStatusText(hDlg,(LPSTR)Temp);  
             }
           break;           
            
         default:
           break;
        }/* end-switch (LOWORD(wParam)) */
    return TRUE;
   }
 return FALSE;
} /* end FilesDlgProc */
   
