'
'  WINFTP.BAS
'
'  WINFTP is a generic PowerBASIC FTP example program that
'  demonstates some (but not all) of the FCE4PB function calls.
'
'  Type WINFTP without arguments for usage.
'

$INCLUDE "FCE32.PBI"
$INCLUDE "KEYCODE.PBI"

GLOBAL ArgVal() As String

$INCLUDE "GETARGS.BAS"

Sub Help()
PRINT "Commands are:"
PRINT "     HELP : Get help"
PRINT "     QUIT : Quit program (Q)"
PRINT "    ASCII : Set ASCII mode"
PRINT "   BINARY : Set BINARY mode"
PRINT "     LIST : List all files"
PRINT " LIST xxx : List file xxx"
PRINT "    NLIST : List file names only"
PRINT "NLIST xxx : List file xxx"
PRINT "      SSD : Set server directory (CWD)"
PRINT "      DSD : Display server directory (PWD)"
PRINT "      MKD : Make server directory"
PRINT "      RMD : Remove server directory"
PRINT "      SLD : Set local directory"
PRINT "      DLD : Display local directory"
PRINT "  GET xxx : Get file xxx"
PRINT "  PUT xxx : Put file xxx"
PRINT "  DEL xxx : Delete file xxx"
PRINT "  PASSIVE : Toggle passive mode"
END Sub

Sub ShowError(BYVAL Code As LONG)
DIM N AS LONG
DIM Temp AS ASCIIZ * 120
Temp = Chr$(0)
N = fceErrorText(0,Code,Temp,120)
PRINT "ERROR ";Code;Left$(Temp, N)
End Sub

Sub ErrorExit(BYVAL Code As LONG)
Call ShowError(Code)
Code = fceClose(0)
Code = fceRelease()
End Sub

'
' CallTheDriver calls fceDriver in a loop until it returns 0,
' indicating that it is done. Refer to the FCE Users Manual
' (FCE4C_U.TXT) for details on using fceDriver.
'
Function CallTheDriver() AS LONG
DIM Code AS LONG
DIM ByteCount AS LONG
DIM LastCount AS LONG
DIM TheTime AS LONG
' get start time
TheTime = TIMER
' AUTO_CALL_DRIVER assumed on (1)
DO
  Code = fceDriver(0)
  IF Code < 0 THEN
     Call ShowError(Code)
     EXIT Function
  END IF
  IF Code <=0 THEN
     EXIT DO
  END IF
  ' display progress
  ByteCount = fceGetInteger(0, %FCE_GET_FILE_BYTES_RCVD)
  IF ByteCount <> LastCount THEN
    LOCATE ScreenY, 1
    PRINT ByteCount;
    LastCount = ByteCount
  END IF
LOOP
' compute run time
TheTime = TIMER - TheTime
LOCATE ScreenY, 1
' re-enable automatic call to seeDriver
Code = fceSetInteger(0, %FCE_SET_AUTO_CALL_DRIVER,1)
CallTheDriver = TheTime
END FUNCTION

Sub Lister(S As String)
Dim N As LONG
Dim L As LONG
Dim X As STRING
X = S
DO
  L = LEN(X)
  N = INSTR(X, CHR$(13))
  IF N > 0 THEN
    PRINT LEFT$(X,N-1)
    X = RIGHT$(X,L-N-1)
    IF L-N <= 2 THEN
      EXIT SUB
    END IF
  ELSE
    EXIT SUB
  END IF
LOOP
End Sub

FUNCTION PbMain() AS LONG
%BUFFER_SIZE = 10000
%TEMP_SIZE = 128
Dim N As Long
Dim X As String
Dim TheCmd As String
Dim TheArg As String
Dim DataBuffer As ASCIIZ * %BUFFER_SIZE
Dim Code As Integer
Dim Version As Integer
Dim Build As Integer
Dim A As String
Dim B As String
Dim C As String
Dim ServerX As ASCIIZ * 50
Dim User AS ASCIIZ * 50
Dim Pass AS ASCIIZ * 50
Dim ServerDir As ASCIIZ  * 100
Dim Temp As Asciiz * %TEMP_SIZE
Dim TheTime AS LONG
Dim ByteCount AS LONG
Dim PassiveMode As Integer
Dim XferMode As Integer

PRINT "WinFTP Version 2.0"

' get command line arguments
N = GetArgs()

IF (N <> 3) THEN
  PRINT "ERROR: Wrong number of arguments"
  PRINT "Usage: WINFTP server user password"
  PRINT "   eg: WINFTP ftp.marshallsoft.com anonymous msc@traveller.com ";
  Exit Function
END IF

' gets ASCIIZ version of argument strings
Pass = ArgVal(3)
User = ArgVal(2)
ServerX = ArgVal(1)
' initialize
PassiveMode = 0
XferMode = ASC("A")

' attach FCE
Code = fceAttach(1, %FCE_KEY_CODE)
IF Code < 0 THEN
  PRINT "Cannot attach FCE" & Str$(Code)
  Return
End If
' display FCE version and build #
Version = fceGetInteger(0, %FCE_GET_VERSION)
Build = fceGetInteger(0, %FCE_GET_BUILD)
C = LTRIM$(Str$(&H0F AND Version))
SHIFT RIGHT Version, 4
B = LTRIM$(Str$(&H0F AND Version))
SHIFT RIGHT Version, 4
A = LTRIM$(Str$(&H0F AND Version))
PRINT "FCE Version " + A + "." + B + "." + C + ", Build" + STR$(Build)
' display registration string
Code = fceGetString(0,%FCE_GET_REGISTRATION, Temp, %TEMP_SIZE)
PRINT Left$(Temp,Code)
' set log file
Code= fceSetString(0,%FCE_SET_LOG_FILE, "WINFTP.LOG")

PRINT "Connecting to ";ServerX;" as user '";User;"' ...";
'''PRINT "Pass=";Pass

Code = fceConnect(0,ServerX,User,Pass)
IF Code < 0 THEN
  Call ShowError(Code)
  Code = fceRelease()
  Exit Function
END IF

PRINT " OK"
PRINT "Ready for commands (type HELP for help)"
' ready for user commands
DO
  PRINT ">";
  LINE INPUT X
  N = INSTR(X, " ")
  IF N > 0 THEN
    TheCmd = LEFT$(X, N - 1)
    TheArg = MID$(X,N+1,LEN(X) - LEN(TheCmd) - 1)
  ELSE
    TheCmd = X
    TheArg = ""
  END IF

  '''PRINT "TheCmd = "; TheCmd
  '''PRINT "TheArg = "; TheArg

  TheCmd = UCASE$(TheCmd)

  ' allow alternate commands
  SELECT CASE TheCmd
    CASE "Q"
      TheCmd = "QUIT"
    CASE "CD"
      TheCmd = "SSD"
    CASE "CWD"
      TheCmd = "SSD"
    CASE "PWD"
      TheCmd = "DSD"
    CASE "LS"
      TheCmd = "LIST"
  END SELECT

  ' execute the command
  Code = 0
  SELECT CASE TheCmd

    CASE "QUIT"
      Code = fceClose(0)
      Code = fceRelease()
      EXIT FUNCTION

    CASE "SSD"
      Temp = TheArg
      IF LEN(TheArg) = 0 THEN
        PRINT "Missing directory name"
        PRINT "EG: SSD MyDir"
      ELSE
        Code = fceSetServerDir(0, Temp)
      END IF

    CASE "DSD"
      Code = fceGetServerDir(0, Temp, %TEMP_SIZE)
      IF Code > 0 THEN
        PRINT LEFT$(Temp,Code)
      END IF

    CASE "SLD"
      Temp = TheArg
      IF LEN(TheArg) = 0 THEN
        PRINT "Missing directory name"
        PRINT "EG: SLD MyDir"
      ELSE
        Code = fceSetLocalDir(0, Temp)
      END IF

    CASE "DLD"
      Code = fceGetLocalDir(0, Temp, %TEMP_SIZE)
      PRINT Left$(Temp,Code)

    CASE "ASCII"
      XferMode = ASC("A")
      Code = fceSetMode(0, XferMode)
      IF Code >= 0 THEN
        PRINT "Xfer mode set to ASCII"
      END IF

    CASE "BINARY"
      XferMode = ASC("B")
      Code = fceSetMode(0, XferMode)
      IF Code >= 0 THEN
        PRINT "Xfer mode set to BINARY"
      END IF

    CASE "LIST"
      IF Len(TheArg) > 0 THEN
        DataBuffer = TheArg
        Code = fceGetList(0, %FCE_FULL_LIST_FILE, DataBuffer, %BUFFER_SIZE)
      Else
        Code = fceGetList(0, %FCE_FULL_LIST, DataBuffer, %BUFFER_SIZE)
      End If
      IF Code > 0 THEN
        Call Lister(LEFT$(DataBuffer,Code))
        Code = fceGetString(0, %FCE_GET_LINE_COUNT, DataBuffer, %BUFFER_SIZE)
        PRINT Str$(Code) + " lines."
      END IF

    CASE "NLIST"
      IF Len(TheArg) > 0 THEN
        DataBuffer = TheArg
        Code = fceGetList(0, %FCE_NAME_LIST_FILE, DataBuffer, %BUFFER_SIZE)
      Else
        Code = fceGetList(0, %FCE_NAME_LIST, DataBuffer, %BUFFER_SIZE)
      End If
      IF Code > 0 THEN
        Call Lister(LEFT$(DataBuffer,Code))
        Code = fceGetString(0, %FCE_GET_LINE_COUNT, DataBuffer, %BUFFER_SIZE)
        PRINT Str$(Code) + " lines."
      END IF

    CASE "HELP"
      Call Help()

    CASE "GET"
      Temp = TheArg
      IF LEN(TheArg) = 0 THEN
        PRINT "Missing filename"
        PRINT "EG: GET MYFILE.TXT"
      ELSE
        PRINT "Downloading file "; Temp;
        IF XferMode = ASC("A") THEN
          PRINT " as ASCII"
        ELSE
          PRINT " as BINARY"
        END IF
        ' disable automatic call to fceDriver
        Code = fceSetInteger(0,%FCE_SET_AUTO_CALL_DRIVER,0)
        ' ask for the file
        Code = fceGetFile(0,Temp)
        IF Code >= 0 THEN
          ' run the driver
          TheTime = CallTheDriver()
          ' display final count
          ByteCount = fceGetInteger(0, %FCE_GET_FILE_BYTES_RCVD)
          PRINT ByteCount;" bytes received in";TheTime;"seconds"
        END IF
      END IF

    CASE "PUT"
      Temp = TheArg
      IF LEN(TheArg) = 0 THEN
        PRINT "Missing filename"
        PRINT "EG: GET MYFILE.TXT"
      ELSE
        PRINT "Uploading file "; Temp;
        IF XferMode = ASC("A") THEN
          PRINT " as ASCII"
        ELSE
          PRINT " as BINARY"
        END IF
        ' disable automatic call to fceDriver
        Code = fceSetInteger(0, %FCE_SET_AUTO_CALL_DRIVER,0)
        ' ask for the file
        Code = fcePutFile(0,Temp)
        IF Code >= 0 THEN
          ' run the driver
          TheTime = CallTheDriver()
          ' display final count
          ByteCount = fceGetInteger(0, %FCE_GET_FILE_BYTES_SENT)
          PRINT ByteCount;" bytes transmitted in";TheTime;"seconds"
        END IF
      END IF

    CASE "DEL"
      Temp = TheArg
      IF LEN(TheArg) = 0 THEN
        PRINT "Missing file name"
        PRINT "EG: DEL MYFILE.TXT"
      ELSE
        Code = fceDelFile(0, Temp)
      END IF

    CASE "PASSIVE"
      PassiveMode = 1 - PassiveMode
      Code = fceSetInteger(0, %FCE_SET_PASSIVE, PassiveMode)
      IF Code > 0 THEN
        IF PassiveMode = 0 THEN
          PRINT "Passive mode disabled"
        ELSE
          PRINT "Passive mode enabled"
        END IF
      END IF

    CASE "MKD"
      Temp = TheArg
      IF LEN(TheArg) = 0 THEN
        PRINT "Missing directory name"
        PRINT "EG: SSD MyDir"
      ELSE
        Code = fceMakeServerDir(0, Temp)
      END IF

    CASE "RMD"
      Temp = TheArg
      IF LEN(TheArg) = 0 THEN
        PRINT "Missing directory name"
        PRINT "EG: SSD MyDir"
      ELSE
        Code = fceDelServerDir(0, Temp)
      END IF

    CASE ELSE
      PRINT "Don't recognize command ";TheCmd
      PRINT "Type HELP for help"
  END SELECT

  ' get error ?
  IF Code < 0 THEN
    Call ShowError(Code)
  END IF

LOOP
END FUNCTION


